@@ -8,13 +8,13 @@ from unittest.mock import _patch, patch
 from warnings import warn
 
 import pyproj
+from orjson import JSONDecodeError
 from owslib.wfs import WebFeatureService
 from owslib.wms import WebMapService
 from requests import Response, Session
 from requests.adapters import HTTPAdapter
 from requests.exceptions import RequestException
 from shapely.geometry import Polygon
-from simplejson import JSONDecodeError
 from urllib3 import Retry
 
 from . import utils
@@ -277,7 +277,7 @@ class ArcGISRESTful:
             geom_query = utils.ESRIGeomQuery(geom, self.out_sr).polygon()
 
         payload = {
-            **geom_query,
+            **geom_query,  # type: ignore
             "spatialRel": self.spatial_relation,
             "returnGeometry": "false",
             "returnIdsOnly": "true",
@@ -410,6 +410,12 @@ class WMSBase:
             _valid_crss = (f"{lyr}: {', '.join(cs)}\n" for lyr, cs in valid_crss.items())
             raise InvalidInputValue("CRS", _valid_crss)
 
+    def get_validlayers(self) -> Dict[str, str]:
+        """Get the layers supportted by the WMS service."""
+        wms = WebMapService(self.url, version=self.version)
+
+        return {wms[lyr].name: wms[lyr].title for lyr in list(wms.contents)}
+
 
 class WMS(WMSBase):
     """Get data from a WMS service within a geometry or bounding box.
@@ -517,7 +523,7 @@ class WMS(WMSBase):
             payload["height"] = _height
             payload["layers"] = lyr
             resp = self.session.get(self.url, payload)
-            return (f"{lyr}_dd_{counter}", resp.content)
+            return f"{lyr}_dd_{counter}", resp.content
 
         return dict(_getmap(i) for i in product(self.layers, bounds))
 
