@@ -4,8 +4,8 @@ from dataclasses import dataclass
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
+import orjson as json
 import pyproj
-import simplejson as json
 from defusedxml import cElementTree as etree
 from requests import Response
 from shapely.geometry import LineString, Point, Polygon, box
@@ -139,7 +139,7 @@ class ESRIGeomQuery:
     geometry: Union[Tuple[float, float], Tuple[float, float, float, float], Polygon]
     wkid: int
 
-    def point(self) -> Dict[str, str]:
+    def point(self) -> Dict[str, Union[str, bytes]]:
         """Query for a point."""
         if len(self.geometry) != 2:
             raise InvalidInputType("geometry (point)", "tuple", "(x, y)")
@@ -148,7 +148,7 @@ class ESRIGeomQuery:
         geo_json = dict(zip(("x", "y"), self.geometry))
         return self.get_payload(geo_type, geo_json)
 
-    def bbox(self) -> Dict[str, str]:
+    def bbox(self) -> Dict[str, Union[str, bytes]]:
         """Query for a bbox."""
         if len(self.geometry) != 4:
             raise InvalidInputType("geometry (bbox)", "tuple", BOX_ORD)
@@ -157,7 +157,7 @@ class ESRIGeomQuery:
         geo_json = dict(zip(("xmin", "ymin", "xmax", "ymax"), self.geometry))
         return self.get_payload(geo_type, geo_json)
 
-    def polygon(self) -> Dict[str, str]:
+    def polygon(self) -> Dict[str, Union[str, bytes]]:
         """Query for a polygon."""
         if not isinstance(self.geometry, Polygon):
             raise InvalidInputType("geomtry", "Shapely's Polygon")
@@ -166,7 +166,7 @@ class ESRIGeomQuery:
         geo_json = {"rings": [[[x, y] for x, y in zip(*self.geometry.exterior.coords.xy)]]}
         return self.get_payload(geo_type, geo_json)
 
-    def get_payload(self, geo_type: str, geo_json: Dict[str, Any]) -> Dict[str, str]:
+    def get_payload(self, geo_type: str, geo_json: Dict[str, Any]) -> Dict[str, Union[str, bytes]]:
         esri_json = json.dumps({**geo_json, "spatialRelference": {"wkid": str(self.wkid)}})
         return {
             "geometryType": geo_type,
@@ -262,7 +262,7 @@ def bbox_decompose(
     resolution: float,
     box_crs: str = DEF_CRS,
     max_px: int = 8000000,
-) -> List[Tuple[Tuple[float, float, float, float], int, int, int]]:
+) -> List[Tuple[Tuple[float, float, float, float], str, int, int]]:
     """Split the bounding box vertically for WMS requests.
 
     Parameters
@@ -286,59 +286,52 @@ def bbox_decompose(
     _bbox = MatchCRS.bounds(bbox, box_crs, DEF_CRS)
     width, height = bbox_resolution(_bbox, resolution, DEF_CRS)
 
-    # Max number of cells that 3DEP can handle safely is about 8 mil.
-    # We need to divide the domain into boxes with cell count of 8 mil.
-    # We fix the height and incremenet the width.
     n_px = width * height
-    if n_px > max_px:
-        geod = pyproj.Geod(ellps="WGS84")
-        west, south, east, north = _bbox
+    if n_px < max_px:
+        return [(bbox, "0_0", width, height)]
 
-        widths = [0]
+    geod = pyproj.Geod(ellps="WGS84")
+    west, south, east, north = _bbox
+
+    def directional_split(az: float, origin: float, dest: float, lvl: float, xy: bool, px: int):
+        divs = [0]
         mul = 1.0
-        while widths[-1] < 1:
+        coords = []
+
+        def get_args(dst, dx):
+            return (dst, lvl, az, dx, 0) if xy else (lvl, dst, az, dx, 1)
+
+        while divs[-1] < 1:
             dim = int(np.sqrt(max_px) * mul)
-            step = dim * resolution
+            step = (dim - 1) * resolution
 
-            # west east decompositon
-            az_x = geod.inv(west, south, east, south)[0]
-            lons = [west]
+            _dest = origin
+            while _dest < dest:
+                args = get_args(_dest, step)
+                coords.append((_dest, geod.fwd(*args[:-1])[args[-1]]))
 
-            while lons[-1] < east:
-                lons.append(geod.fwd(lons[-1], south, az_x, step)[0])
-            lons[-1] = east
+                args = get_args(coords[-1][-1], resolution)
+                _dest = geod.fwd(*args[:-1])[args[-1]]
 
-            nx = len(lons) - 1
-            widths = [dim for _ in range(nx)]
-            widths[-1] = width - (nx - 1) * dim
+            coords[-1] = (coords[-1][0], dest)
+
+            nd = len(coords)
+            divs = [dim for _ in range(nd)]
+            divs[-1] = px - (nd - 1) * dim
             mul -= 0.1
+        return coords, divs
 
-        # south north decompositon
-        heights = [0]
-        mul = 1.0
-        while heights[-1] < 1:
-            dim = int(np.sqrt(max_px) * mul)
-            step = dim * resolution
-            az_y = geod.inv(west, south, west, north)[0]
-            lats = [south]
-            while lats[-1] < north:
-                lats.append(geod.fwd(west, lats[-1], az_y, step)[1])
-            lats[-1] = north
+    az_x = geod.inv(west, south, east, south)[0]
+    lons, widths = directional_split(az_x, west, east, south, True, width)
 
-            ny = len(lats) - 1
-            heights = [dim for _ in range(ny)]
-            heights[-1] = height - (ny - 1) * dim
-            mul -= 0.1
+    az_y = geod.inv(west, south, west, north)[0]
+    lats, heights = directional_split(az_y, south, north, west, False, height)
 
-        bboxs = []
-        counter = 0
-        for bottom, top, h in zip(lats[:-1], lats[1:], heights):
-            for left, right, w in zip(lons[:-1], lons[1:], widths):
-                counter += 1
-                bx_crs = MatchCRS.bounds((left, bottom, right, top), DEF_CRS, box_crs)
-                bboxs.append((bx_crs, counter, w, h))
-    else:
-        bboxs = [(bbox, 1, width, height)]
+    bboxs = []
+    for i, ((bottom, top), h) in enumerate(zip(lats, heights)):
+        for j, ((left, right), w) in enumerate(zip(lons, widths)):
+            bx_crs = MatchCRS.bounds((left, bottom, right, top), DEF_CRS, box_crs)
+            bboxs.append((bx_crs, f"{i}_{j}", w, h))
     return bboxs
 
 
